package com.tzld.piaoquan.recommend.server.service.rank.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.math3.util.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@Slf4j
public class RankStrategy4RegionMergeModelV564 extends RankStrategy4RegionMergeModelBasic {
    @ApolloJsonValue("${rank.score.merge.weightv564:}")
    private Map<String, Double> mergeWeight;

    @Autowired
    private RankStrategy4RegionMergeModelV563 modelV563Service;

    @Override
    public List<Video> mergeAndRankRovRecall(RankParam param) {
        Map<String, Double> mergeWeight = this.mergeWeight != null ? this.mergeWeight : new HashMap<>(0);

        List<Video> result = modelV563Service.mergeAndRankRovRecall(param);
        if (null != result && !result.isEmpty()) {
            try {
                int sortType = mergeWeight.getOrDefault("sortType", 0D).intValue();
                if (sortType > 0) {
                    String scoreKey;
                    if (1 == sortType) {
                        scoreKey = "fmRov";
                    } else {
                        scoreKey = "hasReturnRovScore";
                    }
                    return resort(scoreKey, result);
                }
            } catch (Exception e) {
                log.error("diversity resort error", e);
            }
        }
        return result;
    }

    private List<Video> resort(String scoreKey, List<Video> videos) {
        Map<Long, Video> videoMap = new HashMap<>();
        List<Pair<Long, Double>> list = new ArrayList<>();
        for (Video video : videos) {
            long vid = video.getVideoId();
            videoMap.put(vid, video);

            Map<String, Double> scoreMap = video.getScoresMap();
            list.add(Pair.create(vid, scoreMap.getOrDefault(scoreKey, 0d)));
        }
        list.sort(Comparator.comparingDouble(o -> -o.getSecond()));
        return fillVideo(list, videoMap);
    }

    private List<Video> fillVideo(List<Pair<Long, Double>> list, Map<Long, Video> videoMap) {
        List<Video> result = new ArrayList<>();
        for (Pair<Long, Double> pair : list) {
            Video video = videoMap.get(pair.getFirst());
            result.add(video);
        }
        return result;
    }
}
